﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using System.IO;
using Infragistics.Persistence;
using System.IO.IsolatedStorage;

namespace T_PersistenceManager
{
    public partial class MainPage : UserControl
    {
        private const string PersonlizationFileName = "MyPersonalStore";
        private const long NecessarySpace = 6 * 1024 * 1024; // 6 MB

        public ObservableCollection<Person> People { get; private set; }
        private MemoryStream _defaultSettings;

        public MainPage()
        {
            InitializeComponent();
            _defaultSettings = null;
        }

        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
            People = new ObservableCollection<Person>();

            Person p1 = new Person("Mike", "Fredericks", 24, "Baker");
            p1.Addresses.Add(new Address("Springstreet", 293, 92893, "Hometwon"));
            p1.Addresses.Add(new Address("Wellnesstreet", 109, 32893, "Holidaytown"));

            Person p2 = new Person("Angela", "Michaelson", 49, "Secretary");
            p2.Addresses.Add(new Address("5th Avenue", 483, 555, "New York"));
            p2.Addresses.Add(new Address("Left Lane", 12, 39482, "Small town"));

            Person p3 = new Person("Peter", "Jackson", 56, "Director");
            p3.Addresses.Add(new Address("Hill road", 38, 323, "Los Angeles"));
            p3.Addresses.Add(new Address("River road", 3, 99999, "Hobbiton"));

            People.Add(p1);
            People.Add(p2);
            People.Add(p3);

            PeopleGrid.ItemsSource = People;

            saveAtPersistenceManager(ref _defaultSettings, PeopleGrid);
            loadPersistentSettings(PersonlizationFileName, PeopleGrid);
        }

        private void btnRestoreDefaults_Click(object sender, RoutedEventArgs e)
        {
            if (_defaultSettings != null)
            {
                _defaultSettings.Seek(0, SeekOrigin.Begin);
                loadFromPersistenceManager(_defaultSettings, PeopleGrid);
            }
        }

        private void btnPersonalize_Click(object sender, RoutedEventArgs e)
        {
            using (IsolatedStorageFile iso = IsolatedStorageFile.GetUserStoreForApplication())
            {
                if (iso.FileExists(PersonlizationFileName)) iso.DeleteFile(PersonlizationFileName);

                if (iso.Quota < NecessarySpace)
                { // extend the iso storage space
                    if (!iso.IncreaseQuotaTo(NecessarySpace))
                    {
                        MessageBox.Show("Can not increase storage space");
                        return;
                    }
                }

                MemoryStream memstream = null;
                saveAtPersistenceManager(ref memstream, PeopleGrid);

                writeIntoIsoStorage(PersonlizationFileName, memstream, iso);
            }
        }

        private void btnLoadPersonalization_Click(object sender, RoutedEventArgs e)
        {
            loadPersistentSettings(PersonlizationFileName, PeopleGrid);
        }

        private void writeIntoIsoStorage(string fileName, MemoryStream data, IsolatedStorageFile iso)
        {
            using (IsolatedStorageFileStream stream = new IsolatedStorageFileStream(fileName, FileMode.OpenOrCreate, iso))
            {
                stream.Write(data.ToArray(), 0, (int)data.Length);
            }
        }

        private void loadPersistentSettings(string fileName, DependencyObject obj)
        {
            using (IsolatedStorageFile iso = IsolatedStorageFile.GetUserStoreForApplication())
            {
                if (iso.FileExists(fileName)) loadFromIsoStorage(fileName, obj, iso);
            }
        }

        private void loadFromIsoStorage(string fileName, DependencyObject obj, IsolatedStorageFile iso)
        {
            using (IsolatedStorageFileStream stream = new IsolatedStorageFileStream(fileName, FileMode.Open, iso))
            {
                loadFromPersistenceManager(stream, obj);
            }
        }

        private void loadFromPersistenceManager(Stream stream, DependencyObject obj)
        {
            if (stream == null) return;

            PersistenceManager.Load(obj, stream);
        }

        private void saveAtPersistenceManager(ref MemoryStream stream, DependencyObject obj)
        {
            if (stream == null)
            {
                stream = PersistenceManager.Save(obj);
            }
        }
    }
}
